#include <stdint.h>
#include <stdbool.h>

/* NOTE: example is utilized by running the BLE Scanner application on a smartphone or tablet
 * BLE Scanner can be found here:
 * https://play.google.com/store/apps/details?id=com.macdom.ble.blescanner&hl=en
 *
 * The MCU will update the status of a port and send it over BLE
 * The value of the port can be found in BLE scanner after connecting to the device
 * under "Custom service", first custom characteristic ( UUID of the characteristic ends with "703" )
 * You can also set the value of PORT over your smartphone/tablet, by writing to the characteristic
 * Make sure that the notifications are enabled in your BLE3 Scanner application
 */

sbit BLE3_CTS_MCU at GPIO_PIN3_bit;
sbit BLE3_RTS_MCU at GPIO_PIN28_bit;
sbit BLE3_RST     at GPIO_PIN1_bit;

uint8_t port_read = 0;     // 32 - 39

void main()
{

     GPIO_Digital_Output(&GPIO_PORT_24_31, _GPIO_PINMASK_4 );
     GPIO_Digital_Output(&GPIO_PORT_00_07, _GPIO_PINMASK_1 );
     GPIO_Digital_Input(&GPIO_PORT_00_07, _GPIO_PINMASK_3 );
     GPIO_Digital_Output(&GPIO_PORT_32_39, _GPIO_PINMASK_ALL); // Set GPIO 32-39 port as digital output
     
     Uart2_Init(115200);               // Initialize UART
     delay_ms(500);

     BLE3_RTS_MCU = 0;                 // set RTS pin to 0
     BLE3_RST = 1;                     // pull reset pin high
     delay_ms(4000);
     
     Uart2_Write_Text("AT+UBTLN=");   // set the device name to "mikroE"
     Uart2_Write_Text("\"mikroE\"");
     Uart2_Write_Text("\r");
     delay_ms(2000);

     Uart2_Write_Text("ATO1\r");      // set BLE 3 into data mode, all bytes sent from UART will be sent over BLE
     delay_ms(1000);                  // all bytes received from BLE will be sent to UART
     
     UART2_FCR |= ( 1 << FIFO_EN ) | ( 1 << RCVR_RST );     // set up UART interrupt
     UART2_INT_ENABLE |= ( 1 << RX_AVL_EN );
     IRQ_CTRL &= ~( 1 << GLOBAL_INTERRUPT_MASK );
     
     while(1)
     {
        GPIO_PORT_32_39 = port_read;
        Uart2_Write(port_read);           // send the updated value of the port over BLE
        delay_ms(1000);
        port_read++;
     }

}

// if smartphone/tablet writes to BLE 3, update the port with the value sent from the smartphone/tablet
void UART_RX_ISR() iv IVT_UART2_IRQ ics ICS_AUTO
{
    if( UART2_LSR & ( 1 << DATA_RDY ) )
    {
        port_read = UART2_RBR;
    }
}